ArrayList<Bubble> bubbles = new ArrayList<Bubble>();
int counter = 0;
int dir = 1;
int health = 25;
int score = 0;

boolean gameOver = false;
boolean titleScreen = true;

void setup() {
  size(1280, 720);
  ellipseMode(CENTER);
  textAlign(CENTER, CENTER);
}

void draw() {
  if(titleScreen){
    background(0,0,0);
    textSize(50);
    fill(255);
    text("Bokeh", width / 2, height/2 - 25);
    textSize(25);
    text("Move your mouse around to avoid touching the coloured dots, try to survive for as long as possible \n[ click anywhere to continue ]", width / 2, height/2 + 50);
    textSize(15);
    text("Created by Ty de Bruyn, made using Processing",width/2,height - 40);
  } else {
    background(0.1*counter, 0.075*counter, 0.08*counter);
  
    // Display HUD
    fill(0);
    rect(0,0,width,40);
    rect(0,height-40,width,40);
    textSize(30);
    fill(255);
    textAlign(CENTER, CENTER);
    text("Score: " + score + "                                                          " + "Health: " + health, width/2, 20);
    textAlign(LEFT, CENTER);
    text("Bokeh, a term originating from the Japanese word boke, representing blur or haze, encapsulates a profound aesthetic quality found in the out-of-focus regions within a photograph. It serves as a testament to the intricate interplay between the technical aspects of photography and the artistic vision of the photographer. At its core, bokeh embodies a visual poetry, transforming mundane backgrounds into ethereal canvases that complement and accentuate the primary subject of the image. Its allure lies in its ability to evoke emotion, draw attention, and imbue a sense of depth and dimensionality to an otherwise two-dimensional medium. Photographers often strive to master the manipulation of bokeh, recognizing it as a powerful tool for creative expression. The characteristics of bokeh are heavily influenced by various factors, chief among them being the optical properties of the lens and the settings chosen by the photographer. The shape and size of the aperture blades, the focal length of the lens, and the distance between the subject and the background all play pivotal roles in shaping the final rendition of bokeh in an image. A wide aperture, such as f/1.4 or f/2.8, yields a shallow depth of field, resulting in a pronounced bokeh effect, while narrower apertures produce less prominent bokeh, preserving more detail in the background.The quality of bokeh can be described in terms of its smoothness, texture, and overall aesthetic appeal. Smooth bokeh is characterized by soft, gradual transitions between out-of-focus areas, imparting a dreamlike quality to the image. Conversely, rough or harsh bokeh may exhibit visible artifacts or harsh edges, detracting from the visual harmony of the photograph. Photographers often seek lenses renowned for their ability to produce pleasing bokeh, with certain lens designs and optical coatings engineered specifically to enhance this aspect of image rendering. ANYWAYS... If you made it this far, thank you so much for checking the game out!", width - counter, height - 20);


    // Iterate over the bubbles array, display and move each bubble
    for (Bubble bubble : bubbles) {
      bubble.display();
      bubble.move(bubbles); // Pass the ArrayList of bubbles to the move method
      
      if (!bubble.hitPlayer && bubble.checkCollision()) {
        bubble.c = color(255, 0, 0); // Change the color to red
        bubble.hitPlayer = true;
        health--;
      }
    }
    
    // Add new bubbles every 20 frames
    if (counter % 20 == 0 && dir != 5) {
      bubbles.add(new Bubble(color(random(155, 255), random(155), random(155)), random(40, 100)));
    }
    
    //stop every 300 frames
    if (counter % 300 == 0 && 1 > 2) {
      dir = 5;
    } 
    
    // Change direction randomly every 500 frames
    if (counter % 500 == 0) {
      dir = (int)random(1, 5);
    } 
    
    // gameOver is called after all bubbles have been drawn
    // better looking game over
    if (health <= 0) {
      gameOver();
    }
    
    score = (int) counter / 20;
    counter++;
  }
}

void gameOver() {
  gameOver = true;
  noLoop();
  background(0);
  textSize(100);
  fill(255);
  textAlign(CENTER, CENTER);
  text("Game Over", width / 2, height / 2);
  textSize(25);
  text("Final Score: " + score + "   [ click anywhere to continue ]", width / 2, height / 2 + 60);
}

void restart() {
  // Reset variables
  counter = 0;
  health = 10;
  dir = 1;
  gameOver = false;
  titleScreen = false;
  bubbles.clear(); // Clear the ArrayList of bubbles
  
  // Resume the game
  loop(); // Restart the draw loop
}

void mousePressed(){
  if (gameOver || titleScreen) {
    restart();
  } 
}
void keyPressed() {
  if (key == 'r' || key == 'R') {
    restart(); // Restart the game when 'r' or 'R' is pressed
  }
}

class Bubble {
  color c;
  float xpos, ypos, diameter, xspeed, yspeed;
  
  //  prevents the player from getting hit twice from the same bubble
  boolean hitPlayer = false;
  
  // makes bubbles appear smoother after no longer touching
  int hitBuffer = 0;
  
  Bubble(color tempC, float tempDiameter) {
    c = tempC;
    diameter = tempDiameter;
    
    // bubbles will gradually speed up
    float speedIncrease = counter/500;
    xspeed = 3 + random(-2, 2) + speedIncrease;
    yspeed = 3 + random(-2, 2) + speedIncrease;
    
    switch (dir) {
      case 1:
        xpos = -50;
        ypos = random(height);
        break;
      case 2:
        xpos = width + 50;
        ypos = random(height);
        break;
      case 3:
        xpos = random(width);
        ypos = -50;
        break;
      case 4:
        xpos = random(width);
        ypos = height + 50;
        break;
    }
  }
  
  void display() {
    noStroke();
    fill(c, 90);
    ellipse(xpos, ypos, diameter, diameter);
  }
  
  void move(ArrayList<Bubble> others) {
    boolean touchingOther = false;
    
    // Check for collisions with other bubbles
    if(hitBuffer > 0){
      hitBuffer--;
    } else {
      for (Bubble other : others) {
        if (other != this && dist(xpos, ypos, other.xpos, other.ypos) < (diameter + other.diameter) / 2) {
          touchingOther = true;
          break;
        }
      }
    }
    
    // Adjust speed based on whether touching other bubbles
    float speedFactor = touchingOther ? 0.5 : 1.5; // Adjust as needed
    float adjustedXSpeed = xspeed * speedFactor;
    float adjustedYSpeed = yspeed * speedFactor;
    
    switch (dir) {
      case 1:
        xpos += adjustedXSpeed / (diameter / 50);
        break;
      case 2:
        xpos -= adjustedXSpeed / (diameter / 50);
        break;
      case 3:
        ypos += adjustedYSpeed / (diameter / 50);
        break;
      case 4:
        ypos -= adjustedYSpeed / (diameter / 50);
        break;
      default:
        // Calculate direction towards mouse
        float dx = mouseX - xpos;
        float dy = mouseY - ypos;
        float angle = atan2(dy, dx);
        
        // Move towards mouse
        float speed = 1; // Adjust the speed as needed
        xpos += cos(angle) * speed;
        ypos += sin(angle) * speed;
        break;
    }
  }
  
  boolean checkCollision() {
    float disX = xpos - mouseX;
    float disY = ypos - mouseY;
    return sqrt(sq(disX) + sq(disY)) < diameter / 2;
  }
}
